/*
 * Characters (good and bad)
 */
 
 #ifndef GAMECHAR_INCLUDED
        #define GAMECHAR_INCLUDED

#include "gameobject.h"
#include "manager.h"

class GameCharacter : public SpriteObject {
	protected:
		int animTimer;
		int animFrame;
		int animLength;
		int firstSpriteNum;
		bool canTravelOnWater;
		int deathSpriteNum;
		int deathAnimLength;
	
		Component azimuth; // Degrees
		Component currentSpeed; // Units per 100 ticks. updated frequently.
		Component desiredSpeed;
		Component maxSpeed;
		Component desiredAzimuth;
		Component hitPoints;
		bool killed;
		virtual void runController();
		virtual void setKilled();
		virtual void hitWall(int blockx, int blocky) { }
		void clip();
		void clipPoint(const Vect3d &p, int blockx, int blocky);
		void setSprite();
		bool canPass(const MapBlock &block) {
			if (! canTravelOnWater) {
				if (block.floorType == FloorTypeWater) {
					return false;
				}
			}
			return (block.wallType == WallTypeNone);
		}
		bool isBlocked(int x, int y, int z) {
			Map & map = manager->map;
			return ! canPass(map.blockAt(x,y,z));
		}
		virtual void damage(Component amount) {
			hitPoints -= amount;
			if (hitPoints < 0.01) {
				setKilled();
			}
		}
	public:
		GameCharacter(Manager *m);
		virtual ~GameCharacter();
		virtual void runLogic();
		// Constants
		static const int AnimTicks = 10;
		Component getHitPoints() { return hitPoints; }
	
};
		
// Speed units are: Units per 100 ticks.
// (1 = 1 unit per sec, fairly slow)
// (4 = pretty fast)
const float AzimuthTolerance = 2; // degrees.
const float TurnRate = 5; // degrees per timer tick (100/s)
const float Acceleration = 0.02; // Speed units per tick
const float SpeedTolerance = Acceleration * 2.0;

#endif  // GAMECHAR_INCLUDED
