/*
 * Main file
 *
 * (c) 2005 Mark Robson
 */
#include "timing.h"
#include "manager.h"
#include "constants.h"
#include "graphics_offsets.h"
#include "hud.h"
#include "gameoptions.h"
#include "menus.h"

#include <alleggl.h>
#include <iostream>
#include <cstring>

GameOptions gameOptions;

/*
 * Rendering context states that we will never change.
 */
static void InitGl()
{
	/* remove back faces */
	glEnable(GL_CULL_FACE);
  
	/* speedups */
	glDisable(GL_DITHER);
	glShadeModel(GL_FLAT);
	/*
	 glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
	 glHint(GL_POLYGON_SMOOTH_HINT, GL_FASTEST);
	*/
	glPolygonMode(GL_FRONT, GL_FILL); // GL_FILL or GL_LINE.
}


/*
 * Called at beginning of frame - init projection matrix.
 */
static void InitProjection()
{

	// Set up the 
	glMatrixMode(GL_PROJECTION);
  	glLoadIdentity();
	double aspect = ((double) ScreenWide) / ((double) ScreenHigh);
	// we want 24 units across.
	double height = UnitsWide / aspect;

	glOrtho(- (UnitsWide / 2), (UnitsWide / 2), // left, right
		- (height / 2), (height / 2), // bottom, top
		-100 , 100); // near, far
	glMatrixMode(GL_MODELVIEW);
  	glLoadIdentity();
	// float roottwo = sqrt(2.0);
	glRotatef(-60, 1, 0, 0); // rotate -60 deg about x axis
	glRotatef(45.0, 0, 0, 1); // rotate 45 deg about z.
	// Now sort out the lighting - not used for most objects.
	// Water only.
	// glEnable(GL_LIGHTING);
	static GLfloat LightPosition[] = {6.0f, 0.0f, 6.0f, 0.0f}; // last number 0 = Infinity distance in that dir

	static GLfloat LightAmbient[]  = {0.2f, 0.2f, 0.2f, 1.0f};
	static GLfloat LightDiffuse[]  = {0.8f, 0.8f, 0.8f, 1.0f};
	glLightfv(GL_LIGHT1, GL_AMBIENT, LightAmbient);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse);
	glLightfv(GL_LIGHT1, GL_POSITION, LightPosition);
	glEnable(GL_LIGHT1);
	glShadeModel(GL_SMOOTH);
	glEnable(GL_COLOR_MATERIAL);
}

static void InitAllegro()
{
	allegro_init();
	install_allegro_gl();

	allegro_gl_clear_settings();
	/* allegro_gl_set (AGL_Z_DEPTH, 16); */
	allegro_gl_set (AGL_COLOR_DEPTH, 16);
	allegro_gl_set (AGL_DOUBLEBUFFER, 1);
	// Make fullscreen an option.
	if (gameOptions.fullscreen) {
		allegro_gl_set (AGL_FULLSCREEN, TRUE);	
	} else {
		allegro_gl_set (AGL_WINDOWED, TRUE);	
	}
	allegro_gl_set (AGL_SUGGEST, AGL_COLOR_DEPTH // | AGL_Z_DEPTH
	              | AGL_DOUBLEBUFFER | AGL_WINDOWED);

	set_color_depth (16);
	if (set_gfx_mode(GFX_OPENGL, 640, 480, 0, 0) < 0) {
		throw "Unable to set gfx mode";
	}
	install_keyboard();
	install_mouse();
	InitGl();
}

  
static void InitAll()
{
	InitMilliseconds();
	InitAllegro();
}

/*
static void TestTiming()
{
	std::cout << "ms since start: " << GetMilliseconds() << std::endl;
	for (int i=0; i<5; i++){
		usleep(1000);
		std::cout << "ms since start: " << GetMilliseconds() << std::endl;
	}
}
*/

unsigned int totalticks;
unsigned int totalrenders;

Component mouse_tilex;
Component mouse_tiley;
Component mouse_worldx;
Component mouse_worldy;
Component mouse_screenx;
Component mouse_screeny;

static void HandleInput(Manager &man)
{
	int x = mouse_x;
	int y = mouse_y;
	// Now get them into game units.
	// Offset from centre;
	x -= (ScreenWide / 2);
	y -= (ScreenHigh / 2);
	Component fx = (Component) x;
	Component fy = (Component) y;
	fx = (fx / ScreenWide) * UnitsWide;
	// Assume the same aspect.
	fy = (fy / ScreenWide) * UnitsWide;
	// Invert the Y coordinate because opengl puts 0 at the bottom of
	// The screen, whereas allegro has it at the top.
	fy = - fy;
	mouse_screenx = fx;
	mouse_screeny = fy;
	// multiply them both by sqrt(2)
	fx = fx * RootTwo;
	fy = fy * RootTwo;
	/*
		The following URL has an interestin discussion
		of getting iso coords from a point.
		
		http://nuttybar.drama.uga.edu/pipermail/dirgames-l/2004-January/028053.html
	*/
	Component tx = (fx + 2 * fy) / 2;
      	Component ty = (-fx/2 + fy);
	// Add the centre.
	Vect3d centre = man.getCentre();
	tx += centre.x;
	ty += centre.y;
	// Now find the tile we're in.
	mouse_worldx = tx;
	mouse_worldy = ty;
	tx = floor(tx);
	ty = floor(ty);
	mouse_tilex = tx;
	mouse_tiley = ty;
	if (mouse_b & 1) {
		// Handle click.
		man.handleClick(mouse_worldx, mouse_worldy);
	}
	if ((mouse_b & 2) || key[KEY_LCONTROL] || key[KEY_RCONTROL] )
	{
		// Handle attack command.
		man.handleAttack(mouse_worldx, mouse_worldy);
	}
	if (key[KEY_1] || key[KEY_1_PAD]) {
		man.selectWeapon(1);
	}
	if (key[KEY_2] || key[KEY_2_PAD]) {
		man.selectWeapon(2);
	}
}

// Returns true when we're finished.
static bool DoLogic(Manager &man)
{
	bool finished = false;
	totalticks ++;
	man.runLogic();
	HandleInput(man);
	return finished;
}

static void RenderDebugBits()
{
	// Ensure we're not doing any funky blending or texturing for this.
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
	glColor3f(1,1,1);
	/*
	// Draw a box around the axes
	glBegin(GL_LINE_LOOP);
	glVertex3f(-2, -2,0);
	glVertex3f(2, -2, 0);
	glVertex3f(2, 2,0);
	glVertex3f(-2, 2, 0);
	glEnd();

	// Draw axes
	glBegin(GL_LINES);
	glVertex3f(-2, 0,0);
	glVertex3f(2, 0,0);
	glVertex3f(0, -2, 0);
	glVertex3f(0, 2, 0);
	glVertex3f(0,0, -2);
	glVertex3f(0,0,2);
	glEnd();
	// Draw blobs on the ends of the axes
        glPointSize(15.0);
	glBegin(GL_POINTS);
	glVertex3f(2, 0,0);
	glEnd();
        glPointSize(5.0);
	glBegin(GL_POINTS);
	glVertex3f(0, 2, 0);
	glVertex3f(0,0,2);
	glEnd();
	*/
	
	// Draw the tile where the mouse is.
	
	glBegin(GL_LINE_LOOP);
	glVertex3f(mouse_tilex, mouse_tiley,0);
	glVertex3f(mouse_tilex+1, mouse_tiley, 0);
	glVertex3f(mouse_tilex+1, mouse_tiley+1,0);
	glVertex3f(mouse_tilex, mouse_tiley+1, 0);
	glEnd();

}

// This should be called last. It trashes the modelview matrix.
static void RenderPointer(Manager &man)
{
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND); // Enable Blending
        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	
	glColor3f(1,1,1);
	man.rm.selectSprite(ResourceManager::SpriteSetMisc, ObjectHand);
	// man.rm.selectTexture(0);
	glMatrixMode(GL_MODELVIEW);
  	glLoadIdentity();
	Component xoffset = 0.3;
	glBegin(GL_TRIANGLE_STRIP);
	glTexCoord2d(0.2,0.2);
	glVertex2f(mouse_screenx - 0.5 + xoffset, mouse_screeny + 0.5);
	glTexCoord2d(0.2,0.8);
	glVertex2f(mouse_screenx - 0.5 + xoffset, mouse_screeny - 0.5);
	glTexCoord2d(0.8,0.2);
	glVertex2f(mouse_screenx + 0.5 + xoffset, mouse_screeny + 0.5);
	glTexCoord2d(0.8,0.8);
	glVertex2f(mouse_screenx + 0.5 + xoffset, mouse_screeny - 0.5);
	glEnd();
	/* Debug:
	glDisable(GL_TEXTURE_2D);
	glBegin(GL_POINTS);
	glVertex2f(mouse_screenx , mouse_screeny);
	glEnd();
	*/	
}

// Should be called last.
// Draws a box where the screen would be.
static void RenderDebugBox()
{
	// Cancel out all transformations and rotations etc.
	glMatrixMode(GL_MODELVIEW);
  	glLoadIdentity();

	double aspect = ((double) ScreenWide) / ((double) ScreenHigh);
	double height = UltimateUnitsWide / aspect;

	double halfBoxWidth = UltimateUnitsWide / 2;
	double halfBoxHeight = height / 2;
	glBegin(GL_LINE_LOOP);
	glVertex2f(-halfBoxWidth, -halfBoxHeight);
	glVertex2f(halfBoxWidth, -halfBoxHeight);
	glVertex2f(halfBoxWidth, halfBoxHeight);
	glVertex2f(-halfBoxWidth, halfBoxHeight);
	glEnd();
}

static void DoRender(Manager &man)
{
	// Initialise the GL context projection in case it was in some weird
	// state from last frame's rendering.
	InitProjection();
	// Clear the buffer
	glClearColor(0.0f, 0.0f, 0.5f, 0.0f);         // This Will Clear The Background Color To Blue
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	// Do our interesting rendering here.
	// man.Render sorts out the transformation for the character.
	man.render();
	// Debug bits.
	if (ShowDebugGfx) {
		RenderDebugBits();
	}
	if (ShowDebugBox) {
		RenderDebugBox();
	}
	RenderHud(man);
	RenderPointer(man);
	
	
	// Flip our buffers.
	allegro_gl_flip();
	totalrenders++;
}

static bool ShowPauseMenu(Manager &man)
{
	const char * opts [] = { "Continue", "Quit", 0 };
	int res = ShowMenu(man.rm, "Peril: Paused",
		opts);
	return (res == 1);

}
static bool ShowIntermissionMenu(Manager &man)
{
	const char * opts [] = { "Continue", "Quit", 0 };
	int res = ShowMenu(man.rm, "Peril: Level complete",
		opts);
	return (res == 1);

}
static bool ShowWinMenu(Manager &man)
{
	const char * opts [] = { "Well done", 0 };
	const char * msg = "You Win!";
	if (gameOptions.difficulty == 2) {
		// Unlock hard mode.
		UnlockHardMode();
		// Tell user about it.
		msg = "Hard unlocked";
	}
	int res = ShowMenu(man.rm, msg,
		opts);
	return (res == 1);

}

static void MainLoop(Manager & man)
{
	man.initObjects();

	// Assume that we have to run the logic at least once
	// Before the rendering can be done.
	DoLogic(man);
	// Wait a little while for things to settle before
	// Getting going.
	unsigned int nextticktime = GetMilliseconds() + 500;
	bool finished = false;
	while (! finished) {
		unsigned int now = GetMilliseconds();
		while (now < nextticktime) {
			DoRender(man);
			now = GetMilliseconds();
		}
		if (key[KEY_ESC]) {
			if (ShowPauseMenu(man)) {
				nextticktime = GetMilliseconds() + 500;
			} else {
				finished = true;
			}
		}
		if (man.levelComplete) {
			if (man.levelNum == MaxLevels) {
				ShowWinMenu(man);
				finished = true;
			} else {
				man.levelNum ++;
				if (ShowIntermissionMenu(man)) {
					man.initLevel();
					nextticktime = GetMilliseconds() + 500;
				} else {
					finished = true;
				}
			}
		}
		// if (key[KEY_PAUSE]) paused = true;
		DoLogic(man);
		nextticktime += TickTimeMs;
	}
	std::cout << "total renders: " << totalrenders << std::endl;
	std::cout << "total ticks: " << totalticks << std::endl;
}

void GetOptions(int argc, char *argv[])
{
	gameOptions.fullscreen = false;
	gameOptions.disableHud = false;
	for (int i=1; i<argc; i++) {
		if (std::strcmp(argv[i], "-f") == 0) {
			gameOptions.fullscreen = true;
		}
		if (std::strcmp(argv[i], "-h") == 0) {
			gameOptions.disableHud = true;
		}
	}
}

static void MenuLoop(Manager &man)
{
	do {
		const char * opts [] = { "Start", "Quit", 0 };
		bool unlocked = IsHardUnlocked();
		const char * difficulty_opts [] = { "Too scared to die", "Hurt me lots", 
			unlocked ? "Brown trousers!": "?????", 0 };
		int res = ShowMenu(man.rm, "Peril: Main menu",
			opts);
		if (res == 1) {
			gameOptions.difficulty =
				ShowMenu(man.rm, "Peril: Difficulty", difficulty_opts);
			if ((gameOptions.difficulty != 0) && ((gameOptions.difficulty < 3) || unlocked)) {
				MainLoop(man);
			}
		}
		else 
			return;
	} while (true);
}

int main(int argc, char *argv[])
{
	try {
		GetOptions(argc, argv);
		InitAll();
		// TestTiming();
		Manager man;
		MenuLoop(man);
	} catch (const char *msg) 
	{
		allegro_message ("Unexpected exception:%s\nAllegro error: %s\n"
		                 "Allegro GL error : %s\n",
		                 msg, allegro_error, allegro_gl_error);
	} catch (...) {
		allegro_message ("Unknown exception thrown");
	}
	set_gfx_mode(GFX_TEXT, 0,0, 0, 0);
	return 0;
}

END_OF_MAIN()


