#include "menus.h"
#include "resourcemanager.h"

#include <alleggl.h>
#include <cstring>
#include <fstream>

const int CaptionX = 5;
const int CaptionY = 5;

const int OptionX = 30;
const int OptionY = 120;
const int OptionSpacing = 45;

static void DrawOptions(FONT *f, const char *caption, const char *menuOptions[], int selected)
{
	// Enter normal allegro mode
	allegro_gl_set_allegro_mode();
	clear_to_color(screen, 0);
	// draw caption
	int highcolour = makecol(120,255,120);
	int lowcolour = makecol(40,120,40);
	textout(screen, f, caption,
		CaptionX, CaptionY, highcolour);
	// draw options, highlight selected one.
	int numoptions = 0;
	while (menuOptions[numoptions]) numoptions++;
	for (int i=0; i < numoptions; i++) {
		int colour;
		if ((i + 1) == selected) {
			colour = highcolour;
		} else {
			colour = lowcolour;
		}
		textout(screen, f, menuOptions[i],
			OptionX, OptionY + (i * OptionSpacing), colour);
	}
	// Leave normal allegro mode (enter opengl mode)
	allegro_gl_unset_allegro_mode();
	// Flip buffer.s
	allegro_gl_flip();
}

int ShowMenu(ResourceManager &rm, const char *caption, const char *menuOptions[])
{
	int numoptions = 0;
	while (menuOptions[numoptions]) numoptions++;
	FONT *thefont = rm.getAllegroFont();
	int selected = 1;
	clear_keybuf();
	do {
		DrawOptions(thefont, caption, menuOptions, selected);
		int r = readkey();
		char scancode = (r >> 8) & 0xff;
		char ascii = r & 0xff;
		if (ascii == 27) return 0;
		if (scancode == KEY_UP) {
			if (selected > 1) selected--;
		}
		if (scancode == KEY_DOWN) {
			if (selected < numoptions) selected++;
		}
		if (ascii == 13) {
			return selected;
		}
	} while (1);
}

bool IsHardUnlocked()
{
	std::ifstream testStream("unlocked.txt");
	return (testStream.good());	
}

void UnlockHardMode()
{
	std::ofstream unlocker("unlocked.txt");
	unlocker << "Good luck - you're going to need it";
}


